@@ -733,7 +733,11 @@ convertStringToFormants = function(phonemeString, speaker = 'M1') {
 #' Estimate vocal tract length
 #'
 #' Estimates the length of vocal tract based on formant frequencies, assuming
-#' that the vocal tract can be modeled as a tube open as both ends.
+#' that the vocal tract can be modeled as a tube open as both ends. Algorithm:
+#' first formant dispersion is estimated using the regression method described
+#' in Reby et al. (2005) "Red deer stags use formants as assessment cues during
+#' intrasexual agonistic interactions". The length of vocal tract is then
+#' calculated as speed of sound / 2 / formant dispersion.
 #' @inheritParams getSpectralEnvelope
 #' @param formants a character string like "aaui" referring to default presets
 #'   for speaker "M1"; a vector of formant frequencies; or a list of formant
@@ -811,7 +815,8 @@ estimateVTL = function(formants, speedSound = 35400, checkFormat = TRUE) {
 #' @inheritParams soundgen
 #' @export
 #' @examples
-#' sound = runif(16000)  # white noise
+#' sound = c(rep(0, 1000), runif(16000), rep(0, 1000))  # white noise
+#' # NB: pad with silence to avoid artefacts if removing formants
 #' # playme(sound)
 #' # spectrogram(sound, samplingRate = 16000)
 #'
@@ -938,7 +943,10 @@ addFormants = function(sound,
       )
     )
     # normalize
-    if (normalize) soundFiltered = soundFiltered / max(soundFiltered)
+    if (normalize) {
+      soundFiltered = soundFiltered - mean(soundFiltered)
+      soundFiltered = soundFiltered / max(abs(soundFiltered))
+    }
   }
 
   # remove zero padding
@@ -956,7 +964,7 @@ addFormants = function(sound,
 
 #' Schwa-related formant conversion
 #'
-#' This function performs several types of conceptually related conversion of
+#' This function performs several conceptually related types of conversion of
 #' formant frequencies in relation to the neutral schwa sound based on the
 #' one-tube model of the vocal tract. Case 1: if we know vocal tract length
 #' (VTL) but not formant frequencies, \code{schwa()} estimates formants
@@ -976,7 +984,8 @@ addFormants = function(sound,
 #' vocal tract length and see if each formant is higher or lower than expected
 #' for this vocal tract. For this to work, we have to know either the
 #' frequencies of enough formants (not just the first two) or the true length of
-#' the vocal tract.
+#' the vocal tract. See also \code{\link{estimateVTL}} on the algorithm for
+#' estimating formant dispersion if VTL is not known.
 #' @return Returns a list with the following components: \describe{
 #'   \item{vtl_measured}{VTL as provided by the user, cm}
 #'   \item{vocalTract_apparent}{VTL estimated based on formants frequencies
@@ -987,7 +996,10 @@ addFormants = function(sound,
 #'   user-provided relative formant frequencies, Hz}
 #'   \item{ff_relative}{deviation of formant frequencies from those expected for
 #'   a schwa, \% (e.g. if the first ff_relative is -25, it means that F1 is 25\%
-#' lower than expected for a schwa in this vocal tract)} }
+#'   lower than expected for a schwa in this vocal tract)}
+#'   \item{ff_relative_semitones}{deviation of formant frequencies from those expected for
+#'   a schwa, semitones}
+#' }
 #' @param formants a numeric vector of observed (measured) formant frequencies,
 #'   Hz
 #' @param vocalTract the length of vocal tract, cm
@@ -1011,8 +1023,8 @@ addFormants = function(sound,
 #' # We get an estimate of VTL (s_a$vtl_apparent = 15.2 cm),
 #' #   same as with estimateVTL(formants_a)
 #' # We also get theoretical schwa formants: s_a$ff_schwa
-#' # And we get the difference (%) in observed vs expected
-#' #   formant frequencies: s_a$ff_relative
+#' # And we get the difference (% and semitones) in observed vs expected
+#' #   formant frequencies: s_a[c('ff_relative', 'ff_relative_semitones')]
 #' # [a]: F1 much higher than expected, F2 slightly lower
 #'
 #' formants_i = c(300, 2700, 3400, 4400, 5300, 6400)
@@ -1105,6 +1117,7 @@ schwa = function(formants = NULL,
     }
     ff_schwa = (2 * idx - 1) / 2 * formantDispersion
     ff_relative = (formants / ff_schwa - 1) * 100
+    ff_relative_semitones = HzToSemitones(formants) - HzToSemitones(ff_schwa)
     ff_theoretical = NULL
   } else {
     ## we know formants_relative and vocalTract and want to convert ff to Hz
@@ -1122,6 +1135,7 @@ schwa = function(formants = NULL,
     ff_theoretical = ff_schwa * (1 + formants_relative / 100)
     vocalTract_apparent = NULL
     ff_relative = formants_relative
+    ff_relative_semitones = HzToSemitones(ff_theoretical) - HzToSemitones(ff_schwa)
   }
 
   # prepare the output
@@ -1130,7 +1144,8 @@ schwa = function(formants = NULL,
              ff_measured = formants,
              ff_schwa = ff_schwa,
              ff_theoretical = ff_theoretical,
-             ff_relative = ff_relative)
+             ff_relative = ff_relative,
+             ff_relative_semitones = ff_relative_semitones)
   # do not return empty elements
   out = out[lapply(out, length) > 0]
   return(out)
